
import webpay.client.Webpay;
import java.io.*;
import java.lang.*;
import java.util.*;
import java.security.*;

/**
 * Class used for testing the behaviour of the Webpay java client (webpay.client.Webpay)
 * Demonstrates the following:
 *
 * - Overriding default server list and port
 * - Constructing the Webpay object
 * - Building the transaction bundle
 * - Executing the transaction
 * - Polling on a status of "IN PROGRESS"
 * - Catching exceptions
 */

public class BaseTest
{
	public static void main ( String[] args )
	{
		String responseCode = null;
		try
		{

			// Check that the necessary arguments are present. The last two are optional.
			if ( args.length < 6 )
			{
				System.out.println(
					"Usage: BaseTest [certificate file] [certificate password] " +
					"[clientid] [cardnumber] [value as XX.XX] [card expiry " +"date as XXXX] ([servers]) ([port]) " );
				System.exit( 1 );
			}

			// Construct the webpay object. The parameters are clientid, certpath, cert_passphrase
			Webpay webpay = null;
			try
			{
				webpay = new Webpay( args[2] , args[0], args[1] );
			}
			catch (Exception wt_ex )
			{
				System.err.println( "Error creating the Webpay object" );
				wt_ex.printStackTrace();
			}

			// If server list is present in argument list, override the default server
			if ( args.length > 6 )
			{
				String[] servers = matrix.tools.StringTools.split( args[6], ',' );
				webpay.setServers( servers );
			}

			// If port is present in argument list, override the default port
			if ( args.length > 7 )
				webpay.setPort( args[7] );

			// Put other arguments into Webpay object
			webpay.put( "CARDEXPIRYDATE", args[5] );
			webpay.put( "CARDDATA", args[3] );
			webpay.put( "TOTALAMOUNT", args[4] );
			webpay.put( "TRANSACTIONTYPE", "PURCHASE" );
			webpay.put( "INTERFACE", "creditcard" );
			webpay.put( "TAXAMOUNT", "0.00" );
			webpay.put( "TERMINALTYPE", "0" );

			// Execute the transaction.
			try
			{
				webpay.execute();
			}
			catch ( IOException io_ex )
			{
				// The transaction has failed at a protocol level.
				System.err.println( "Error executing transaction." );
				io_ex.printStackTrace();
				// If TXNREFERENCE has been set, then the transaction was initialised
				// before failing. Let the transaction continue to see if it brings a
				// result back.
				if ( webpay.get( "TXNREFERENCE" ) != null )
				{
					responseCode = "IP";
				}
			}

			// No exception has occurred, so the transaction has returned a
			// response code. Check to see if the transaction is still "IN PROGRESS".
			// If it is, poll server with status requests until either the response
			// code changes or to a maximum of three status requests.
			if ( responseCode == null ) responseCode = webpay.get( "RESPONSECODE" );
			for ( int statusCheckCount = 0; statusCheckCount < 3; statusCheckCount++ )
			{
				if ( "IP".equals( responseCode ) )
				{
					// The transaction is still in progress. Send a status request.
					webpay.put( "TRANSACTIONTYPE", "STATUS" );
					try
					{
						Thread.sleep( 500 ); // Half-second polling delay
						webpay.execute();
					}
					catch ( IOException io_ex2 )
					{
						// The transaction has failed at a protocol level.
						System.err.println( "Error executing transaction." );
						io_ex2.printStackTrace();
					}
					responseCode = webpay.get( "RESPONSECODE" );
				}
				else
				{
					// Break out of for loop, we don't need poll anymore as the transaction
					// is no longer IN PROGRESS
					statusCheckCount = 3;
				}
			}

			// We have finished polling server.
			if ( "IP".equals( responseCode ) )
			{
				// The transaction was still in progress after three iterations through
				// the polling loop. Kill the session.
				System.out.println( "The server failed to respond in a timely fashion." );

				System.out.println( "Try sending transaction again." );
				System.exit( 0 );
			}
			else
			{
				// The transaction has completed. Display results.
				System.out.println ( "Transaction Reference: " +
					webpay.get ( "TXNREFERENCE" ) );
				System.out.println ( "Response Code: " + webpay.get ( "RESPONSECODE" ) );
				System.out.println ( "Response Text: " + webpay.get ( "RESPONSETEXT" ) );
				System.out.println ( "Authorisation Code: " + webpay.get ( "AUTHCODE" ) );
				System.out.println ( "Error Message: " + webpay.get ( "ERROR" ) );
			}
		}
		catch ( Exception e )
		{
			// General exception caught.
			System.out.println ( "Exception caught." );
			e.printStackTrace();
		}
	}
}

